@@ -20,14 +20,19 @@ for handler in Logger.handlers:
 	if not isinstance(handler, ConsoleHandler):
 		handler.setFormatter(formatter)
 
-resource_add_path(__path__[0] + "/assets")
+resource_add_path(__path__[0])
+for submodule in [
+	"/assets",
+	"/assets/rltiles",
+	"/assets/kenney1bit",
+]:
+	resource_add_path(__path__[0] + submodule)
 
 __all__ = [
 	"graph",
 	"grid",
 	"app",
 	"card",
-	"dialog",
 	"game",
 	"menu",
 	"spritebuilder",
