@@ -17,16 +17,30 @@
 import json
 import os
 import shutil
-from functools import partial
+from collections import defaultdict
+from functools import cached_property, partial
 from threading import Thread
+from typing import Callable
+from zipfile import ZIP_DEFLATED, ZipFile
 
 from lisien.exc import OutOfTimelineError
 
+from .charsview import CharactersScreen
+from .logview import LogScreen
+from .menu import MainMenuScreen
+from .rulesview import CharacterRulesScreen, RulesScreen
+from .screen import MainScreen
+from .spritebuilder import PawnConfigScreen, SpotConfigScreen
+from .statcfg import StatScreen
+from .stores import FuncsEdScreen, StringsEdScreen
+from .timestream import TimestreamScreen
+
 if "KIVY_NO_ARGS" not in os.environ:
 	os.environ["KIVY_NO_ARGS"] = "1"
 
 from kivy.app import App
 from kivy.clock import Clock, triggered
+from kivy.lang import Builder
 from kivy.logger import Logger
 from kivy.properties import (
 	AliasProperty,
@@ -35,23 +49,9 @@ from kivy.properties import (
 	ObjectProperty,
 	StringProperty,
 )
-from kivy.resources import resource_add_path, resource_find
+from kivy.resources import resource_find
 from kivy.uix.screenmanager import NoTransition, Screen, ScreenManager
 
-import elide
-import elide.charsview
-import elide.dialog
-import elide.menu
-import elide.rulesview
-import elide.screen
-import elide.spritebuilder
-import elide.statcfg
-import elide.stores
-import elide.timestream
-from elide.graph.arrow import GraphArrow
-from elide.graph.board import GraphBoard
-from elide.grid.board import GridBoard
-from elide.util import load_kv, logwrap
 from lisien.proxy import (
 	CharacterProxy,
 	CharStatProxy,
@@ -60,9 +60,10 @@ from lisien.proxy import (
 	ThingProxy,
 )
 
-resource_add_path(elide.__path__[0] + "/assets")
-resource_add_path(elide.__path__[0] + "/assets/rltiles")
-resource_add_path(elide.__path__[0] + "/assets/kenney1bit")
+from .graph.arrow import GraphArrow
+from .graph.board import GraphBoard
+from .grid.board import GridBoard
+from .util import devour, logwrap
 
 
 def trigger(func):
@@ -93,6 +94,87 @@ class ElideApp(App):
 	workers = NumericProperty(None, allownone=True)
 	immediate_start = BooleanProperty(False)
 	character_name = ObjectProperty()
+	closed = BooleanProperty(True)
+	stopped = BooleanProperty(False)
+
+	@cached_property
+	def _bindings(self) -> dict[tuple, set[int]]:
+		class OnceSet(set):
+			def add(self, __element):
+				if __element in self:
+					raise RuntimeError("Already bound", __element)
+				super().add(__element)
+
+		return defaultdict(OnceSet)
+
+	@cached_property
+	def _unbinders(self) -> list[Callable[[], None]]:
+		return [self.unbind_all]
+
+	def unbind_all(self):
+		Logger.debug("ElideApp: unbinding everything")
+		for uid in devour(self._bindings["ElideApp", "character"]):
+			self.unbind_uid("character", uid)
+		for uid in devour(self._bindings["ElideApp", "character_name"]):
+			self.unbind_uid("character_name", uid)
+		for uid in devour(
+			self._bindings["CharactersScreen", "character_name"]
+		):
+			self.chars.unbind_uid("character_name", uid)
+		for uid in devour(self._bindings["ElideApp", "selected_proxy"]):
+			self.unbind_uid("selected_proxy", uid)
+		for uid in devour(self._bindings["MainScreen", "statlist"]):
+			self.mainscreen.unbind_uid("statlist", uid)
+		for uid in devour(self._bindings["ElideApp", "selection"]):
+			self.unbind_uid("selection", uid)
+		if hasattr(self, "mainscreen"):
+			for graphboard in self.mainscreen.graphboards:
+				for uid in devour(
+					self._bindings["GraphBoard", graphboard, "selection"]
+				):
+					self.mainscreen.graphboards[graphboard].unbind_uid(
+						"selection", uid
+					)
+			for gridboard in self.mainscreen.gridboards:
+				for uid in devour(
+					self._bindings["GridBoard", gridboard, "selection"]
+				):
+					self.mainscreen.gridboards[gridboard].unbind_uid(
+						"selection", uid
+					)
+
+	@cached_property
+	def _togglers(self):
+		return {}
+
+	def _get_games_path(self):
+		return os.path.join(self.prefix, self.games_dir)
+
+	def _set_games_path(self, str_or_pair: str | tuple[str, str]):
+		if isinstance(str_or_pair, str):
+			a, b = os.path.split(str_or_pair)
+		else:
+			a, b = str_or_pair
+		self.prefix, self.games_dir = a, b
+
+	games_path = AliasProperty(
+		_get_games_path, _set_games_path, bind=("prefix", "games_dir")
+	)
+
+	def _get_play_dir(self):
+		return os.path.join(self.prefix, self.game_name)
+
+	def _set_play_dir(self, str_or_pair: str | tuple[str, str]):
+		if isinstance(str_or_pair, str):
+			*a, b = str.split(os.path.sep)
+			a = os.path.sep.join(a)
+		else:
+			a, b = str_or_pair
+		self.prefix, self.game_name = a, b
+
+	play_path = AliasProperty(
+		_get_play_dir, _set_play_dir, bind=("prefix", "game_name")
+	)
 
 	@logwrap
 	def on_selection(self, *_):
@@ -116,7 +198,7 @@ class ElideApp(App):
 			str(origin.name) + "->" + str(destination.name)
 		)
 
-	def on_character_name(self, name, *_):
+	def on_character_name(self, _, name):
 		if (
 			hasattr(self, "engine")
 			and name in self.engine.character
@@ -224,10 +306,8 @@ class ElideApp(App):
 		if char == self.character:
 			return
 		if char.name not in self.mainscreen.graphboards:
-			load_kv("elide.graph.board")
 			self.mainscreen.graphboards[char.name] = GraphBoard(character=char)
 		if char.name not in self.mainscreen.gridboards:
-			load_kv("elide.grid.board")
 			self.mainscreen.gridboards[char.name] = GridBoard(character=char)
 		self.character = char
 		self.selected_proxy = self._get_selected_proxy()
@@ -292,12 +372,13 @@ class ElideApp(App):
 			pdb.set_trace()
 
 		self.manager = ScreenManager(transition=NoTransition())
+		print(f"created screen manager with id {id(self.manager)}")
 		if config["elide"]["inspector"] == "yes":
 			from kivy.core.window import Window
 			from kivy.modules import inspector
 
 			inspector.create_inspector(Window, self.manager)
-		self.mainmenu = elide.menu.MainMenuScreen(toggle=self._toggler("main"))
+		self.mainmenu = MainMenuScreen(toggle=self.toggler("main"))
 		self.manager.add_widget(self.mainmenu)
 		if self.immediate_start:
 			self.start_game()
@@ -322,7 +403,7 @@ class ElideApp(App):
 		self.branch, self.turn, self.tick = now
 		self.mainscreen.ids.turnscroll.value = self.turn
 
-	def start_subprocess(self, path=None, *_):
+	def start_subprocess(self, path=None, archive_path=None, *_):
 		"""Start the lisien core and get a proxy to it
 
 		Must be called before ``init_board``
@@ -331,6 +412,7 @@ class ElideApp(App):
 		Logger.debug(f"ElideApp: start_subprocess(path={path!r})")
 		if hasattr(self, "procman") and hasattr(self.procman, "engine_proxy"):
 			raise ChildProcessError("Subprocess already running")
+		self.closed = False
 		config = self.config
 		enkw = {
 			"do_game_start": getattr(self, "do_game_start", False),
@@ -368,7 +450,12 @@ class ElideApp(App):
 		self.procman = EngineProcessManager(
 			use_thread=self.use_thread,
 		)
-		self.engine = engine = self.procman.start(path, **enkw)
+		if archive_path is None:
+			self.engine = engine = self.procman.start(path, **enkw)
+		else:
+			self.engine = engine = self.procman.load_archive(
+				archive_path, path, **enkw
+			)
 		Logger.debug("Got EngineProxy")
 		if "boardchar" in engine.eternal:
 			self.character_name = engine.eternal["boardchar"]
@@ -377,6 +464,10 @@ class ElideApp(App):
 				self.character = engine.character[self.character_name]
 			else:
 				self.character = engine.new_character(self.character_name)
+		elif engine.character:
+			self.character = next(iter(engine.character.values()))
+		else:
+			self.character = engine.new_character("physical")
 		Logger.debug("Pulled character")
 		self.pull_time()
 		Logger.debug("Pulled time")
@@ -390,22 +481,21 @@ class ElideApp(App):
 
 	trigger_start_subprocess = trigger(start_subprocess)
 
+	@logwrap
 	def init_board(self, *_):
 		"""Get the board widgets initialized to display the game state
 
 		Must be called after start_subprocess
 
 		"""
-		Logger.debug("ElideApp: init_board")
 		self.chars.names = char_names = list(self.engine.character)
+		Logger.debug(f"ElideApp: making grid boards for: {char_names}")
 		for name in char_names:
 			if name not in self.mainscreen.graphboards:
-				load_kv("elide.graph.board")
 				self.mainscreen.graphboards[name] = GraphBoard(
 					character=self.engine.character[name]
 				)
 			if name not in self.mainscreen.gridboards:
-				load_kv("elide.grid.board")
 				self.mainscreen.gridboards[name] = GridBoard(
 					character=self.engine.character[name]
 				)
@@ -414,18 +504,26 @@ class ElideApp(App):
 				self.engine.character[self.engine.eternal["boardchar"]]
 			)
 
-	def _toggler(self, screenname):
+	def toggler(self, screenname):
+		"""Return a function that shows or hides a named screen"""
+		if screenname in self._togglers:
+			return self._togglers[screenname]
+
 		def tog(*_):
 			if self.manager.current == screenname:
+				Logger.debug("ElideApp: toggling back to mainscreen")
 				self.manager.current = "mainscreen"
 			else:
+				Logger.debug(f"ElideApp: toggling to {screenname}")
 				self.manager.current = screenname
 
+		self._togglers[screenname] = tog
+
 		return tog
 
 	def _add_screens(self, *_):
 		Logger.debug("ElideApp: _add_screens")
-		toggler = self._toggler
+		toggler = self.toggler
 		config = self.config
 
 		pawndata = json.loads(config["elide"]["thing_graphics"])
@@ -435,8 +533,7 @@ class ElideApp(App):
 				["Custom pawns", "custom_pawn_imgs/custom.atlas"]
 			] + pawndata
 
-		load_kv("elide.spritebuilder")
-		self.pawncfg = elide.spritebuilder.PawnConfigScreen(
+		self.pawncfg = PawnConfigScreen(
 			toggle=toggler("pawncfg"),
 			data=pawndata,
 		)
@@ -447,59 +544,57 @@ class ElideApp(App):
 			spotdata = [
 				["Custom spots", "custom_spot_imgs/custom.atlas"]
 			] + spotdata
-		self.spotcfg = elide.spritebuilder.SpotConfigScreen(
+		self.spotcfg = SpotConfigScreen(
 			toggle=toggler("spotcfg"),
 			data=spotdata,
 		)
-
-		load_kv("elide.statcfg")
-		self.statcfg = elide.statcfg.StatScreen(toggle=toggler("statcfg"))
-		load_kv("elide.rulesview")
-		self.rules = elide.rulesview.RulesScreen(toggle=toggler("rules"))
-
-		self.charrules = elide.rulesview.CharacterRulesScreen(
+		for builder in (
+			self.pawncfg.ids.dialog.ids.builder.__ref__(),
+			self.spotcfg.ids.dialog.ids.builder.__ref__(),
+		):
+			self._bindings["SpriteBuilder", id(builder), "data"].add(
+				builder.fbind("data", builder._trigger_update)
+			)
+			self._unbinders.append(builder.unbind_all)
+			builder.update()
+		self.statcfg = StatScreen(toggle=toggler("statcfg"))
+		self.rules = RulesScreen(toggle=toggler("rules"))
+		self.charrules = CharacterRulesScreen(
 			character=self.character, toggle=toggler("charrules")
 		)
-		self.bind(character=self.charrules.setter("character"))
-		load_kv("elide.charsview")
-		self.chars = elide.charsview.CharactersScreen(
+		self._bindings["ElideApp", "character"].add(
+			self.fbind("character", self.charrules.setter("character"))
+		)
+		Logger.debug("ElideApp: bound charrules setter")
+		self.chars = CharactersScreen(
 			toggle=toggler("chars"), new_board=self.new_board
 		)
-		self.bind(character_name=self.chars.setter("character_name"))
-
-		def chars_push_character_name(*_):
-			self.unbind(character_name=self.chars.setter("character_name"))
-			self.character_name = self.chars.character_name
-			self.bind(character_name=self.chars.setter("character_name"))
-
-		self.chars.push_character_name = chars_push_character_name
-		load_kv("elide.stores")
-		self.strings = elide.stores.StringsEdScreen(toggle=toggler("strings"))
-
-		self.funcs = elide.stores.FuncsEdScreen(
-			name="funcs", toggle=toggler("funcs")
+		self._bindings["ElideApp", "character_name"].add(
+			self.fbind("character_name", self.chars.setter("character_name"))
 		)
-
-		self.bind(selected_proxy=self.statcfg.setter("proxy"))
-		load_kv("elide.timestream")
-		self.timestream = elide.timestream.TimestreamScreen(
+		self.strings = StringsEdScreen(toggle=toggler("strings"))
+		self.funcs = FuncsEdScreen(name="funcs", toggle=toggler("funcs"))
+		self._bindings["ElideApp", "selected_proxy"].add(
+			self.fbind("selected_proxy", self.statcfg.setter("proxy"))
+		)
+		self.timestream = TimestreamScreen(
 			name="timestream", toggle=toggler("timestream")
 		)
-		load_kv("elide.screen")
-		load_kv("elide.charmenu")
-		load_kv("elide.statcfg")
-		load_kv("elide.stepper")
-		load_kv("elide.dialog")
-		self.mainscreen = elide.screen.MainScreen(
+		self.log_screen = LogScreen(name="log", toggle=toggler("log"))
+		self.mainscreen = MainScreen(
 			use_kv=config["elide"]["user_kv"] == "yes",
 			play_speed=int(config["elide"]["play_speed"]),
 		)
 		if self.mainscreen.statlist:
 			self.statcfg.statlist = self.mainscreen.statlist
-		self.mainscreen.bind(statlist=self.statcfg.setter("statlist"))
-		self.bind(
-			selection=self.refresh_selected_proxy,
-			character=self.refresh_selected_proxy,
+		self._bindings["MainScreen", "statlist"].add(
+			self.mainscreen.fbind("statlist", self.statcfg.setter("statlist"))
+		)
+		self._bindings["ElideApp", "selection"].add(
+			self.fbind("selection", self.refresh_selected_proxy)
+		)
+		self._bindings["ElideApp", "character"].add(
+			self.fbind("character", self.refresh_selected_proxy)
 		)
 		for wid in (
 			self.mainscreen,
@@ -512,6 +607,7 @@ class ElideApp(App):
 			self.strings,
 			self.funcs,
 			self.timestream,
+			self.log_screen,
 		):
 			self.manager.add_widget(wid)
 
@@ -542,8 +638,9 @@ class ElideApp(App):
 				)
 				continue
 			self.manager.remove_widget(wid)
+		self.manager.current = "main"
 
-	def start_game(self, *_, name=None, cb=None):
+	def start_game(self, *_, name=None, archive_path=None, cb=None):
 		Logger.debug(f"ElideApp: start_game(name={name!r}, cb={cb!r})")
 		if hasattr(self, "engine"):
 			Logger.error("Already started the game")
@@ -551,14 +648,19 @@ class ElideApp(App):
 		game_name = name or self.game_name
 		if self.game_name != game_name:
 			self.game_name = game_name
-		gamedir = os.path.join(self.prefix, self.games_dir, game_name)
+		os.makedirs(
+			self.play_path,
+			exist_ok=True,
+		)
 		self._add_screens()
-		self.manager.current = "mainscreen"
-		engine = self.engine = self.start_subprocess(gamedir)
-		self.mainscreen.populate()
+		engine = self.engine = self.start_subprocess(
+			self.play_path, archive_path
+		)
 		self.init_board()
+		self.mainscreen.populate()
 		if cb:
 			cb()
+		self.manager.current = "mainscreen"
 		return engine
 
 	def close_game(self, *_, cb=None):
@@ -575,23 +677,43 @@ class ElideApp(App):
 			return  # already closed
 
 		self._copy_log_files()
-		game_wd = os.path.join(self.prefix, self.game_name)
-		pycache = os.path.join(self.prefix, self.game_name, "__pycache__")
+		pycache = os.path.join(self.play_path, "__pycache__")
 		if os.path.exists(pycache):
 			shutil.rmtree(pycache)
-		archived = shutil.make_archive(
-			self.game_name, "zip", str(game_wd), logger=Logger
-		)
-		archived_base = os.path.basename(archived)
-		os.makedirs(self.games_dir, exist_ok=True)
-		if os.path.exists(os.path.join(self.games_dir, archived_base)):
-			os.remove(os.path.join(self.games_dir, archived_base))
-		os.rename(archived, os.path.join(self.games_dir, archived_base))
+		archived_base = self.game_name + ".zip"
+		os.makedirs(self.games_path, exist_ok=True)
+		archived_abs = str(os.path.join(self.games_path, archived_base))
+		if os.path.exists(archived_abs):
+			os.remove(archived_abs)
+		with ZipFile(archived_abs, "x", ZIP_DEFLATED) as zf:
+			for fn in os.listdir(self.play_path):
+				if os.path.isdir(os.path.join(self.play_path, fn)):
+					for fnn in os.listdir(os.path.join(self.play_path, fn)):
+						if os.path.isdir(
+							os.path.join(self.play_path, fn, fnn)
+						):
+							for pqfn in os.listdir(
+								os.path.join(self.play_path, fn, fnn)
+							):
+								zf.write(
+									os.path.join(
+										self.play_path, fn, fnn, pqfn
+									),
+									os.path.join(fn, fnn, pqfn),
+								)
+						else:
+							zf.write(
+								os.path.join(self.play_path, fn, fnn),
+								os.path.join(fn, fnn),
+							)
+				else:
+					zf.write(os.path.join(self.play_path, fn), fn)
 		if not hasattr(self, "leave_game"):
-			shutil.rmtree(game_wd)
+			shutil.rmtree(self.play_path)
 		self._remove_screens()
 		if cb:
 			cb()
+		self.closed = True
 
 	def update_calendar(self, calendar, past_turns=1, future_turns=5):
 		"""Fill in a calendar widget with actual simulation data"""
@@ -669,18 +791,28 @@ class ElideApp(App):
 		if self.character_name != self.character.name:
 			self.character_name = self.character.name
 		if hasattr(self, "_oldchar"):
-			self.mainscreen.graphboards[self._oldchar.name].unbind(
-				selection=self.setter("selection")
-			)
-			self.mainscreen.gridboards[self._oldchar.name].unbind(
-				selection=self.setter("selection")
-			)
+			for uid in devour(
+				self._bindings["GraphBoard", self._oldchar.name, "selection"]
+			):
+				self.mainscreen.graphboards[self._oldchar.name].unbind_uid(
+					"selection", uid
+				)
+			for uid in devour(
+				self._bindings["GridBoard", self._oldchar.name, "selection"]
+			):
+				self.mainscreen.gridboards[self._oldchar.name].unbind_uid(
+					"selection", uid
+				)
 		self.selection = None
-		self.mainscreen.graphboards[self.character.name].bind(
-			selection=self.setter("selection")
+		self._bindings["GraphBoard", self.character.name, "selection"].add(
+			self.mainscreen.graphboards[self.character.name].fbind(
+				"selection", self.setter("selection")
+			)
 		)
-		self.mainscreen.gridboards[self.character.name].bind(
-			selection=self.setter("selection")
+		self._bindings["GridBoard", self.character.name, "selection"].add(
+			self.mainscreen.gridboards[self.character.name].fbind(
+				"selection", self.setter("selection")
+			)
 		)
 
 	def copy_to_shared_storage(
@@ -799,16 +931,32 @@ class ElideApp(App):
 
 	def on_stop(self, *largs):
 		"""Sync the database, wrap up the game, and halt."""
-		Logger.debug("ElideApp: stopping")
-		if hasattr(self, "stopped"):
+		if self.stopped:
 			return
-		self.funcs.save()
+		Logger.debug("ElideApp: stopping")
+		if hasattr(self, "funcs"):
+			self.funcs.save()
 		if hasattr(self, "engine"):
-			self.close_game()
-		self.stopped = True
-		Logger.debug("ElideApp: stopped")
+			self.close_game(cb=partial(self.setter("stopped"), 0.0, True))
+		else:
+			self.stopped = True
+		for unbinder in self._unbinders:
+			unbinder()
+		for k, v in self._bindings.items():
+			if v:
+				raise RuntimeError("Still bound", k, v)
+		for loaded_kv in Builder.files[:]:
+			if not loaded_kv.endswith("/kivy/data/style.kv"):
+				Builder.unload_file(loaded_kv)
+				Logger.debug(f"ElideApp: unloaded {loaded_kv}")
+			else:
+				Logger.debug(f"ElideApp: won't unload {loaded_kv}")
 		return True
 
+	def on_stopped(self, *_):
+		if self.stopped:
+			Logger.debug("ElideApp: stopped")
+
 	def delete_selection(self):
 		"""Delete both the selected widget and whatever it represents."""
 		Logger.debug("ElideApp: delete_selection")
