@@ -19,7 +19,7 @@ from operator import attrgetter
 from kivy.app import App
 from kivy.clock import Clock, triggered
 from kivy.logger import Logger
-from kivy.properties import ObjectProperty, StringProperty
+from kivy.properties import NumericProperty, ObjectProperty, StringProperty
 from kivy.uix.boxlayout import BoxLayout
 from kivy.uix.button import Button
 from kivy.uix.floatlayout import FloatLayout
@@ -34,7 +34,7 @@ from kivy.uix.widget import Widget
 
 from .card import Card, DeckBuilderScrollBar, DeckBuilderView
 from .stores import FuncEditor
-from .util import logwrap, store_kv
+from .util import devour, load_kv, logwrap
 
 
 def trigger(func):
@@ -44,8 +44,7 @@ def trigger(func):
 dbg = Logger.debug
 
 
-# How do these get instantiated?
-class RuleButton(ToggleButton, RecycleDataViewBehavior):
+class RuleButton(ToggleButton):
 	"""A button to select a rule to edit"""
 
 	rulesview = ObjectProperty()
@@ -58,11 +57,34 @@ class RuleButton(ToggleButton, RecycleDataViewBehavior):
 		# This really ought to be done with the selection behavior
 		if self.state == "down":
 			self.rulesview.rule = self.rule
-			for button in self.ruleslist.children[0].children:
+			for button in self.ruleslist.iter_rule_buttons():
 				if button != self:
 					button.state = "normal"
 
 
+class RuleButtonBox(BoxLayout, RecycleDataViewBehavior):
+	rulesview = ObjectProperty()
+	ruleslist = ObjectProperty()
+	rule = ObjectProperty()
+	button = ObjectProperty()
+	index = NumericProperty()
+
+	@trigger
+	def move_rule_up(self, *_):
+		i = int(self.index)
+		if i <= 0:
+			return
+		self.ruleslist.rulebook.move_rule_back(i)
+
+	@trigger
+	def move_rule_down(self, *_):
+		i = int(self.index)
+		if i >= len(self.ruleslist.rulebook):
+			return
+		self.ruleslist.rulebook.move_rule_forward(i)
+		self.index = i + 1
+
+
 class RulesList(RecycleView):
 	"""A list of rules you might want to edit
 
@@ -78,14 +100,25 @@ class RulesList(RecycleView):
 		"""Make sure to update when the rulebook changes"""
 		if self.rulebook is None:
 			return
+		Logger.debug("RulesList: got rulebook")
 		self.rulebook.connect(self._trigger_redata, weak=False)
 		self.redata()
 
+	@logwrap(section="RulesList")
+	def on_rulesview(self, *_):
+		if self.rulebook is None:
+			Logger.debug("RulesList: got rulesview before rulebook")
+			return
+		self.redata()
+
 	@logwrap(section="RulesList")
 	def redata(self, *_):
 		"""Make my data represent what's in my rulebook right now"""
 		if self.rulesview is None:
-			Clock.schedule_once(self.redata, 0)
+			Logger.debug(
+				"RulesList: delaying redata until rulesview available"
+			)
+			self._trigger_redata()
 			return
 		data = [
 			{
@@ -104,6 +137,12 @@ class RulesList(RecycleView):
 			Clock.unschedule(self._scheduled_redata)
 		self._scheduled_redata = Clock.schedule_once(self.redata, 0)
 
+	def iter_rule_buttons(self):
+		if not self.children:
+			return
+		for rules_box in self.children[0].children:
+			yield rules_box.button
+
 
 class RulesView(Widget):
 	"""The view to edit a rule
@@ -145,6 +184,17 @@ class RulesView(Widget):
 		super().__init__(**kwargs)
 		self.finalize()
 
+	def unbind_all(self, *_):
+		binds = App.get_running_app()._bindings
+		for att in ("size", "pos"):
+			for uid in devour(binds["RulesView", att]):
+				self.unbind_uid(att, uid)
+		for functyp in "trigger", "prereq", "action":
+			for uid in devour(binds["RulesView", "rule", functyp]):
+				self.unbind_uid("rule", uid)
+			for uid in devour(binds["RulesView", functyp, "decks"]):
+				getattr(self, f"_{functyp}_builder").unbind_uid("decks", uid)
+
 	@logwrap(section="RulesView")
 	def finalize(self, *_):
 		"""Add my tabs"""
@@ -152,6 +202,8 @@ class RulesView(Widget):
 		if not self.canvas:
 			Clock.schedule_once(self.finalize, 0)
 			return
+		app = App.get_running_app()
+		binds = app._bindings
 
 		deck_builder_kwargs = {
 			"pos_hint": {"x": 0, "y": 0},
@@ -164,7 +216,12 @@ class RulesView(Widget):
 		self._tabs = TabbedPanel(
 			size=self.size, pos=self.pos, do_default_tab=False
 		)
-		self.bind(size=self._tabs.setter("size"), pos=self._tabs.setter("pos"))
+		for att in ("size", "pos"):
+			for uid in devour(binds["RulesView", att]):
+				self.unbind_uid(att, uid)
+			binds["RulesView", att].add(
+				self.fbind(att, self._tabs.setter(att))
+			)
 		self.add_widget(self._tabs)
 
 		for functyp in "trigger", "prereq", "action":
@@ -173,13 +230,17 @@ class RulesView(Widget):
 			self._tabs.add_widget(getattr(self, "_{}_tab".format(functyp)))
 			builder = DeckBuilderView(**deck_builder_kwargs)
 			setattr(self, "_{}_builder".format(functyp), builder)
+			for uid in devour(binds["RulesView", functyp, "decks"]):
+				builder.unbind_uid("decks", uid)
+			uid = builder.fbind(
+				"decks", getattr(self, f"_trigger_push_{functyp}s")
+			)
 			setattr(
 				self,
 				f"_trigger_push_{functyp}s_uid",
-				builder.fbind(
-					"decks", getattr(self, f"_trigger_push_{functyp}s")
-				),
+				uid,
 			)
+			binds["RulesView", functyp, "decks"].add(uid)
 			scroll_left = DeckBuilderScrollBar(
 				size_hint_x=0.01,
 				pos_hint={"x": 0, "y": 0},
@@ -216,7 +277,12 @@ class RulesView(Widget):
 					size_hint=(None, None),
 				)
 			)
-			self.bind(rule=getattr(self, "_trigger_pull_{}s".format(functyp)))
+			for uid in devour(binds["RulesView", "rule", functyp]):
+				self.unbind_uid("rule", uid)
+			binds["RulesView", "rule", functyp].add(
+				self.fbind("rule", getattr(self, f"_trigger_pull_{functyp}s"))
+			)
+		app._unbinders.append(self.unbind_all)
 		self._finalized = True
 
 	@logwrap(section="RulesView")
@@ -338,6 +404,10 @@ class RulesView(Widget):
 	@logwrap(section="RulesView")
 	def pull_triggers(self, *args):
 		"""Refresh the cards in the trigger builder"""
+		binds = App.get_running_app()._bindings
+		binds["RulesView", "trigger", "decks"].remove(
+			self._trigger_push_triggers_uid
+		)
 		self._trigger_builder.unbind_uid(
 			"decks", self._trigger_push_triggers_uid
 		)
@@ -345,12 +415,19 @@ class RulesView(Widget):
 		self._trigger_push_triggers_uid = self._trigger_builder.fbind(
 			"decks", self._trigger_push_triggers
 		)
+		binds["RulesView", "trigger", "decks"].add(
+			self._trigger_push_triggers_uid
+		)
 
 	_trigger_pull_triggers = trigger(pull_triggers)
 
 	@logwrap(section="RulesView")
 	def pull_prereqs(self, *args):
 		"""Refresh the cards in the prereq builder"""
+		binds = App.get_running_app()._bindings
+		binds["RulesView", "prereq", "decks"].remove(
+			self._trigger_push_prereqs_uid
+		)
 		self._prereq_builder.unbind_uid(
 			"decks", self._trigger_push_prereqs_uid
 		)
@@ -360,12 +437,19 @@ class RulesView(Widget):
 		self._trigger_push_prereqs_uid = self._prereq_builder.fbind(
 			"decks", self._trigger_push_prereqs
 		)
+		binds["RulesView", "prereq", "decks"].add(
+			self._trigger_push_prereqs_uid
+		)
 
 	_trigger_pull_prereqs = trigger(pull_prereqs)
 
 	@logwrap(section="RulesView")
 	def pull_actions(self, *args):
 		"""Refresh the cards in the action builder"""
+		binds = App.get_running_app()._bindings
+		binds["RulesView", "action", "decks"].remove(
+			self._trigger_push_actions_uid
+		)
 		self._action_builder.unbind_uid(
 			"decks", self._trigger_push_actions_uid
 		)
@@ -375,6 +459,9 @@ class RulesView(Widget):
 		self._trigger_push_actions_uid = self._action_builder.fbind(
 			"decks", self._trigger_push_actions
 		)
+		binds["RulesView", "action", "decks"].add(
+			self._trigger_push_actions_uid
+		)
 
 	_trigger_pull_actions = trigger(pull_actions)
 
@@ -420,9 +507,11 @@ class RulesView(Widget):
 		:param what: a string, 'trigger', 'prereq', or 'action'
 
 		"""
+		binds = App.get_running_app()._bindings
 		builder = getattr(self, "_{}_builder".format(what))
 		updtrig = getattr(self, "_trigger_upd_unused_{}s".format(what))
-		builder.unbind(decks=updtrig)
+		while binds["RulesView", "builder", what]:
+			self.unbind_uid("decks", binds[RulesView, "builder", what].pop())
 		funcs = OrderedDict()
 		cards = list(self._action_builder.decks[1])
 		cards.reverse()
@@ -434,7 +523,7 @@ class RulesView(Widget):
 		unused = list(funcs.values())
 		unused.reverse()
 		builder.decks[1] = unused
-		builder.bind(decks=updtrig)
+		binds["RulesView", "builder", what].add(builder.bind("decks", updtrig))
 
 	@logwrap(section="RulesView")
 	def upd_unused_actions(self, *_):
@@ -507,16 +596,30 @@ class RulesBox(BoxLayout):
 	ruleslist = ObjectProperty()
 	rulesview = ObjectProperty()
 
+	def __init__(self, **kwargs):
+		super().__init__(**kwargs)
+		App.get_running_app()._unbinders.append(self.unbind_all)
+
 	@property
 	def engine(self):
 		return App.get_running_app().engine
 
+	def unbind_all(self):
+		binds = App.get_running_app()._bindings
+		for uid in devour(binds["RulesBox", "ruleslist", "children"]):
+			self.unbind_uid("children", uid)
+
 	@logwrap(section="RulesBox")
 	def on_ruleslist(self, *_):
 		if not self.ruleslist.children:
 			Clock.schedule_once(self.on_ruleslist, 0)
 			return
-		self.ruleslist.children[0].bind(children=self._upd_ruleslist_selection)
+		binds = App.get_running_app()._bindings[
+			"RulesBox", "ruleslist", "children"
+		]
+		while binds:
+			self.unbind_uid("children", binds.pop())
+		binds.add(self.fbind("children", self._upd_ruleslist_selection))
 
 	@logwrap(section="RulesBox")
 	def new_rule(self, *_):
@@ -534,7 +637,7 @@ class RulesBox(BoxLayout):
 	def _upd_ruleslist_selection(self, *_):
 		if not hasattr(self, "_new_rule_name"):
 			return
-		for child in self.ruleslist.children[0].children:
+		for child in self.ruleslist.iter_rule_buttons():
 			if child.text == self._new_rule_name:
 				child.state = "down"
 			else:
@@ -548,11 +651,17 @@ class RulesScreen(Screen):
 	rulebook = ObjectProperty()
 	toggle = ObjectProperty()
 	rulesview = ObjectProperty()
+	box = ObjectProperty()
 
 	@property
 	def engine(self):
 		return App.get_running_app().engine
 
+	def __init__(self, **kw):
+		load_kv("rulesview.kv")
+		load_kv("card.kv")
+		super().__init__(**kw)
+
 	@logwrap(section="RulesScreen")
 	def new_rule(self, *_):
 		self.children[0].new_rule()
@@ -564,6 +673,11 @@ class CharacterRulesScreen(Screen):
 	character = ObjectProperty()
 	toggle = ObjectProperty()
 
+	def __init__(self, **kw):
+		load_kv("rulesview.kv")
+		load_kv("card.kv")
+		super().__init__(**kw)
+
 	@logwrap(section="CharacterRulesScreen")
 	def _get_rulebook(self, rb):
 		return {
@@ -621,68 +735,3 @@ class CharacterRulesScreen(Screen):
 		# in elide, so I don't need to account for that, but what if the
 		# rulebook changes as a result of some code running in the lisien core?
 		# 2018-08-13
-
-
-store_kv(
-	__name__,
-	"""
-<RuleButton>:
-    text: self.rule.name if self.rule else ''
-<RulesList>:
-    viewclass: 'RuleButton'
-    SelectableRecycleBoxLayout:
-        default_size: None, dp(56)
-        default_size_hint: 1, None
-        height: self.minimum_height
-        size_hint_y: None
-        orientation: 'vertical'
-<RulesBox>:
-    new_rule_name: rulename.text
-    ruleslist: ruleslist
-    rulesview: rulesview
-    rulebook_name: str(self.rulebook.name) if self.rulebook is not None else ''
-    entity_name: str(self.entity.name) if self.entity is not None else ''
-    orientation: 'vertical'
-    Label:
-        text: root.entity_name + '    -    ' + root.rulebook_name
-        size_hint_y: 0.05
-    BoxLayout:
-        orientation: 'horizontal'
-        RulesList:
-            id: ruleslist
-            rulebook: root.rulebook
-            entity: root.entity
-            rulesview: rulesview
-            size_hint_x: 0.2
-        RulesView:
-            id: rulesview
-            rulebook: root.rulebook
-            entity: root.entity
-            size_hint_x: 0.8
-    BoxLayout:
-        orientation: 'horizontal'
-        size_hint_y: 0.07
-        TextInput:
-            id: rulename
-            hint_text: 'New rule name'
-            write_tab: False
-            multiline: False
-        Button:
-            text: '+'
-            on_release: root.new_rule()
-        Button:
-            id: closebut
-            text: 'Close'
-            on_release: root.toggle()
-<RulesScreen>:
-    name: 'rules'
-    rulesview: box.rulesview
-    RulesBox:
-        id: box
-        rulebook: root.rulebook
-        entity: root.entity
-        toggle: root.toggle
-<CharacterRulesScreen>:
-    name: 'charrules'
-""",
-)
