@@ -46,7 +46,7 @@ from kivy.uix.screenmanager import Screen
 from kivy.uix.textinput import TextInput
 from kivy.uix.togglebutton import ToggleButton
 
-from .util import logwrap, store_kv
+from .util import devour, load_kv, logwrap
 
 
 def trigger(func):
@@ -112,6 +112,14 @@ class StoreList(RecycleView):
 		self._i2name = {}
 		self._name2i = {}
 		super().__init__(**kwargs)
+		App.get_running_app()._unbinders.append(self.unbind_all)
+
+	def unbind_all(self):
+		binds = App.get_running_app()._bindings
+		for uid in devour(
+			binds["StoreList", self.store._store, "boxl", "selected_nodes"]
+		):
+			self.boxl.unbind_uid("selected_nodes", uid)
 
 	@logwrap(section="RecycleToggleButton")
 	def on_store(self, *_):
@@ -120,7 +128,18 @@ class StoreList(RecycleView):
 
 	@logwrap(section="RecycleToggleButton")
 	def on_boxl(self, *_):
-		self.boxl.bind(selected_nodes=self._pull_selection)
+		if self.store is None:
+			Clock.schedule_once(self.on_boxl, 0)
+			return
+		app = App.get_running_app()
+		if not app:
+			return
+		binds = app._bindings[
+			"StoreList", self.store._store, "boxl", "selected_nodes"
+		]
+		for uid in devour(binds):
+			self.boxl.unbind_uid("selected_nodes", uid)
+		binds.add(self.boxl.fbind("selected_nodes", self._pull_selection))
 
 	@logwrap(section="RecycleToggleButton")
 	def _pull_selection(self, *_):
@@ -204,6 +223,10 @@ class StringsEdScreen(Screen):
 	edbox = ObjectProperty()
 	"""Widget containing editors for the current string and its name"""
 
+	def __init__(self, **kw):
+		load_kv("stores.kv")
+		super().__init__(**kw)
+
 	@logwrap(section="RecycleToggleButton")
 	def on_language(self, *_):
 		if self.edbox is None:
@@ -325,12 +348,23 @@ class StringInput(Editor):
 	validate_name_input = ObjectProperty()
 	"""Boolean function for checking if a string name is acceptable"""
 
+	def unbind_all(self):
+		for uid in devour(
+			App.get_running_app()._bindings["StringInput", "name_wid", "text"]
+		):
+			self.name_wid.unbind_uid("text", uid)
+
 	@logwrap(section="StringInput")
 	def on_name_wid(self, *_):
 		if not self.validate_name_input:
 			Clock.schedule_once(self.on_name_wid, 0)
 			return
-		self.name_wid.bind(text=self.validate_name_input)
+		app = App.get_running_app()
+		binds = app._bindings
+		binds["StringInput", "name_wid", "text"].add(
+			self.name_wid.fbind("text", self.validate_name_input)
+		)
+		app._unbinders.append(self.unbind_all)
 
 	@logwrap(section="StringInput")
 	def _get_name(self):
@@ -392,10 +426,30 @@ class EdBox(BoxLayout):
 			Clock.schedule_once(self.on_store_name, 0)
 			return
 		self.store = getattr(app.engine, self.store_name)
+		app._unbinders.append(self.unbind_all)
+
+	def unbind_all(self):
+		binds = App.get_running_app()._bindings
+		for uid in devour(
+			binds["EdBox", self.store_name, "storelist", "selection_name"]
+		):
+			self.storelist.unbind_uid("selection_name", uid)
 
 	@logwrap(section="EdBox")
 	def on_storelist(self, *_):
-		self.storelist.bind(selection_name=self.setter("selection_name"))
+		if not self.store_name:
+			Clock.schedule_once(self.on_storelist, 0)
+			return
+		binds = App.get_running_app()._bindings[
+			"EdBox", self.store_name, "storelist", "selection_name"
+		]
+		for uid in devour(binds):
+			self.storelist.unbind_uid("selection_name", uid)
+		binds.add(
+			self.storelist.fbind(
+				"selection_name", self.setter("selection_name")
+			)
+		)
 
 	@trigger
 	@logwrap(section="EdBox")
@@ -567,26 +621,46 @@ class FuncEditor(Editor):
 	"""Instance of ``StoreList`` that shows all the functions you can edit"""
 	codeinput = ObjectProperty()
 	params = ListProperty(["obj"])
+	params_disabled = BooleanProperty(True)
+	params_text = StringProperty()
 	name = StringProperty()
 	_text = StringProperty()
 	_do_parse = True
 
+	def on_params_text(self, *_):
+		if "params" not in self.ids:
+			Clock.schedule_once(self.on_params_text, 0)
+			return
+		self.ids.params.text = self.params_text
+
 	def _get_source(self):
-		code = self.get_default_text(self.name)
+		code = self.get_default_text(self.name or self.name_wid.text)
 		if self._text:
 			code += indent(self._text, " " * 4)
 		else:
 			code += " " * 4 + "pass"
 		return code.rstrip(" \n\t")
 
+	def unbind_all(self):
+		for uid in devour(
+			App.get_running_app()._bindings["FuncEditor", "codeinput", "text"]
+		):
+			self.codeinput.unbind_uid("text", uid)
+
 	@logwrap(section="FuncEditor")
 	def _set_source(self, v):
 		if not self.codeinput:
 			Clock.schedule_once(partial(self._set_source, v), 0)
 			return
-		self.codeinput.unbind(text=self.setter("_text"))
+		binds = App.get_running_app()._bindings
+		while binds["FuncEditor", "codeinput", "text"]:
+			self.codeinput.unbind_uid(
+				"text", binds["FuncEditor", "codeinput", "text"].pop()
+			)
 		self.params, self.codeinput.text = munge_source(str(v))
-		self.codeinput.bind(text=self.setter("_text"))
+		binds["FuncEditor", "codeinput", "text"].add(
+			self.codeinput.fbind("text", self.setter("_text"))
+		)
 
 	source = AliasProperty(_get_source, _set_source, bind=("_text", "params"))
 
@@ -596,8 +670,30 @@ class FuncEditor(Editor):
 		return "def {}({}):\n".format(name, ", ".join(self.params))
 
 	def on_codeinput(self, *args):
+		app = App.get_running_app()
+		app._unbinders.append(self.unbind_all)
+		binds = app._bindings["FuncEditor", "codeinput", "text"]
 		self._text = self.codeinput.text
-		self.codeinput.bind(text=self.setter("_text"))
+		while binds:
+			self.codeinput.unbind_uid("text", binds.pop())
+		binds.add(self.codeinput.fbind("text", self.setter("_text")))
+
+
+class MethodEditor(FuncEditor):
+	def on_params_text(self, *_):
+		params = self.params_text.split(", ")
+		if self._validate_params(params):
+			self.params = params
+
+	@staticmethod
+	def _validate_params(params: list[str]) -> bool:
+		return params[0] == "self"
+
+
+class FunctionEditor(MethodEditor):
+	@staticmethod
+	def _validate_params(params: list[str]) -> bool:
+		return True
 
 
 class FuncsEdBox(EdBox):
@@ -638,213 +734,12 @@ class FuncsEdScreen(Screen):
 
 	toggle = ObjectProperty()
 
+	def __init__(self, **kw):
+		load_kv("stores.kv")
+		super().__init__(**kw)
+
 	@logwrap(section="FuncsEdScreen")
 	def save(self, *args):
 		self.ids.triggers.save()
 		self.ids.prereqs.save()
 		self.ids.actions.save()
-
-
-store_kv(
-	__name__,
-	"""
-#: import py3lexer pygments.lexers.Python3Lexer
-<StoreButton>:
-	size_hint_y: None
-	height: 30
-<StoreList>:
-	viewclass: 'StoreButton'
-	boxl: boxl
-	SelectableRecycleBoxLayout:
-		id: boxl
-		default_size: None, dp(56)
-		default_size_hint: 1, None
-		height: self.minimum_height
-		size_hint_y: None
-		orientation: 'vertical'
-<StringInput>:
-	name_wid: stringname
-	orientation: 'vertical'
-	BoxLayout:
-		size_hint_y: 0.07
-		Label:
-			id: title
-			text: 'Title: '
-			size_hint_x: None
-			width: self.texture_size[0]
-		StringNameInput:
-			id: stringname
-			multiline: False
-			write_tab: False
-			_trigger_save: root._trigger_save
-		Button:
-			text: 'del'
-			size_hint_x: 0.1
-			on_release: root._trigger_delete()
-	TextInput:
-		id: string
-		disabled: root.disable_text_input
-<StringsEdBox>:
-	editor: strings_ed
-	storelist: strings_list
-	orientation: 'vertical'
-	BoxLayout:
-		orientation: 'horizontal'
-		StoreList:
-			id: strings_list
-			size_hint_x: 0.2
-			store: root.store
-		StringInput:
-			id: strings_ed
-			store: root.store
-			disable_text_input: root.disable_text_input
-			validate_name_input: root.validate_name_input
-			_trigger_save: root._trigger_save
-			_trigger_delete: root._trigger_delete
-<StringsEdScreen>:
-	name: 'strings'
-	edbox: edbox
-	BoxLayout:
-		orientation: 'vertical'
-		StringsEdBox:
-			id: edbox
-			toggle: root.toggle
-			store_name: 'string'
-			language: root.language
-		BoxLayout:
-			size_hint_y: 0.05
-			Label:
-				text_size: self.size
-				halign: 'right'
-				valign: 'middle'
-				text: 'Language: '
-				size_hint_x: 0.2
-			LanguageInput:
-				id: language
-				screen: root
-				hint_text: root.language
-				write_tab: False
-				multiline: False
-			Button:
-				text: 'Close'
-				on_release: edbox.dismiss()
-<Py3CodeInput@CodeInput>:
-	lexer: py3lexer()
-<FuncEditor>:
-	codeinput: code
-	name_wid: funname
-	orientation: 'vertical'
-	BoxLayout:
-		orientation: 'horizontal'
-		size_hint_y: None
-		height: funname.height
-		Py3CodeInput:
-			id: imafunction
-			text: 'def'
-			disabled: True
-			size_hint: (None, None)
-			height: self.line_height + self.font_size
-			width: self.font_size * len(self.text)
-			background_disabled_normal: ''
-			disabled_foreground_color: self.foreground_color
-		FunctionNameInput:
-			id: funname
-			size_hint: (0.4, None)
-			height: self.line_height + self.font_size
-			multiline: False
-			write_tab: False
-			_trigger_save: root._trigger_save
-			on_text: root.validate_name_input(self.text)
-			hint_text: root.name
-		Py3CodeInput:
-			id: params
-			text: '({}):'.format(', '.join(root.params))
-			disabled: True
-			size_hint_y: None
-			height: self.line_height + self.font_size
-			background_disabled_normal: ''
-			disabled_foreground_color: self.foreground_color
-		Button:
-			text: 'del'
-			size_hint_x: 0.1 if not self.disabled else 0.0
-			background_disabled_normal: ''
-			background_disabled_down: ''
-			disabled_color: (0., 0., 0., 0.)
-			background_disabled_color: (0., 0., 0., 0.)
-			on_release: root._trigger_delete()
-			disabled: not root.deletable
-	BoxLayout:
-		orientation: 'horizontal'
-		Label:
-			canvas:
-				Color:
-					rgba: params.background_color
-				Rectangle:
-					pos: self.pos
-					size: self.size
-				Color:
-					rgba: [1., 1., 1., 1.]
-			# PEP8 standard indentation width is 4 spaces
-			text: ' ' * 4
-			size_hint_x: None
-			width: self.texture_size[0]
-		Py3CodeInput:
-			id: code
-			disabled: root.disable_text_input
-<FuncsEdBox>:
-	editor: funcs_ed
-	storelist: funcs_list
-	orientation: 'vertical'
-	data: [(item['name'], self.store.get_source(item['name'])) for item in funcs_list.data[1:]]
-	BoxLayout:
-		orientation: 'horizontal'
-		BoxLayout:
-			orientation: 'vertical'
-			size_hint_x: 0.2
-			StoreList:
-				id: funcs_list
-				store: root.store
-		FuncEditor:
-			id: funcs_ed
-			name: root.selection_name
-			store: root.store
-			storelist: funcs_list
-			disable_text_input: root.disable_text_input
-			validate_name_input: root.validate_name_input
-			_trigger_save: root._trigger_save
-			_trigger_delete: root._trigger_delete
-	BoxLayout:
-		size_hint_y: 0.07
-		Button:
-			text: 'Close'
-			on_release: root.dismiss()
-<FuncsEdScreen>:
-	name: 'funcs'
-	TabbedPanel:
-		default_tab: action
-		TabbedPanelItem:
-			id: trigger
-			text: 'Trigger'
-			on_state: triggers.save()
-			FuncsEdBox:
-				id: triggers
-				toggle: root.toggle
-				store_name: 'trigger'
-		TabbedPanelItem:
-			id: prereq
-			text: 'Prereq'
-			on_state: prereqs.save()
-			FuncsEdBox:
-				id: prereqs
-				toggle: root.toggle
-				store_name: 'prereq'
-		TabbedPanelItem:
-			id: action
-			text: 'Action'
-			on_state: actions.save()
-			FuncsEdBox:
-				id: actions
-				toggle: root.toggle
-				store_name: 'action'
-""",
-)
